import Config from './config.js';

document.querySelector('html').setAttribute('lang', browser.i18n.getUILanguage());

/**
 * @param {string} url
 * @returns {Promise<any>}
 */
async function fetchJson(url) {
	const response = await fetch(url);
	return response.json();
}

/**
 * @param {number} days
 * @returns {Promise<Array<object>>}
 */
function fetchCalendar(days = 28) {
	let url = `${Config.calenderUrl}?o=json`;
	if (days) {
		url += `&r=${days}`;
	}
	return fetchJson(url);
}

/**
 * @returns {Promise<object>}
 */
function fetchSpaceApi() {
	return fetchJson(Config.spaceApiUrl);
}

async function fetchNewData() {
	try {
		const result = await Promise.allSettled([fetchCalendar(), fetchSpaceApi()]);
		const calendarJson = result[0].value;
		const spaceApiJson = result[1].value;

		if (spaceApiJson) {
			const now = new Date();
			const eventActive = calendarJson?.some((event) => (
				new Date(event.begin) <= now && now <= new Date(event.end)
			)) ?? false;

			const badgeText = browser.i18n.getMessage(spaceApiJson.state.open ? 'badgeOpen' : eventActive ? 'badgeEvent' : 'badgeClosed');
			const badgeBgColor = spaceApiJson.state.open ? Config.openColor : eventActive ? Config.eventColor : Config.closedColor;
			await browser.browserAction.setBadgeText({text: badgeText});
			await browser.browserAction.setBadgeBackgroundColor({color: badgeBgColor});

			const {spaceApi} = await browser.storage.local.get('spaceApi');
			if (spaceApi && spaceApi.state.open !== spaceApiJson.state.open) {
				const state = browser.i18n.getMessage(spaceApiJson.state.open ? 'open' : 'closed');
				await browser.notifications.create('status-change', {
					type: 'basic',
					title: browser.i18n.getMessage('stateNotificationTitle'),
					message: browser.i18n.getMessage('stateNotificationMessage', state),
					iconUrl: browser.runtime.getURL('icons/favicon.svg'),
				});
			}
		}

		await browser.storage.local.set({
			calendar: calendarJson,
			spaceApi: spaceApiJson,
		});
	} catch (error) {
		console.error(error);
	}
}

browser.runtime.onStartup.addListener(async () => {
	await browser.storage.local.remove(['calendar', 'spaceApi']);
});

/**
 * @param {Alarm} alarm
 */
function handleAlarm(alarm) {
	if (alarm.name === 'fetchData') {
		fetchNewData();
	}
}

function startFetching() {
	fetchNewData();

	browser.alarms.onAlarm.addListener(handleAlarm);

	const now = new Date();
	now.setMinutes(Math.ceil((now.getMinutes() + 1) / Config.refreshTimeout) * Config.refreshTimeout, 0, 0);
	browser.alarms.create('fetchData', {
		when: now.getTime(),
		periodInMinutes: Config.refreshTimeout,
	});
}

window.addEventListener('offline', async () => {
	browser.alarms.onAlarm.removeListener(handleAlarm);
	await browser.alarms.clear('fetchData');
});
window.addEventListener('online', () => {
	startFetching();
});
if (window.navigator.onLine) {
	startFetching();
}
